import pygame
import sprite
import math


class Room(object):
    
    def __init__(self, bg, walk, fg):
        self.background = sprite.Sprite(bg)
        walkimg = pygame.image.load(walk)
        try:
            self.walk = pygame.PixelArray(walkimg)
            self.true = lambda x: x
        except:
            try:
                self.walk = pygame.surfarray.pixels2d(walkimg)
                self.true = lambda x: x
            except:
                self.walk = [[walkimg.get_at([x, y]) for y in \
                 range(walkimg.get_height())]
                 for x in range(walkimg.get_width())]
                self.true = lambda x: x[3]
        self.foreground = sprite.Sprite(fg)
        self.objects = []
        self.paths = []

    def add(self, object):
        self.objects.append(object)

    def remove(self, object):
        if object in self.objects:
            self.objects.remove(object)
    
    def draw(self):
        if hasattr(self, "event"):
            self.event.time -= 1
            if self.event.time <= 0:
                self.event()
                del self.event
        self.animate_paths()
        self.background.draw()
        self.objects.sort(key=lambda o: o.pos[1])
        if hasattr(self, "perspective"):
            for ob in self.objects:
                i = ob.image
                sc = self.perspective*ob.pos[1]/480.0
                ob.image = pygame.transform.rotozoom(ob.image,
                        0,sc)
                off = ob.offset[:]
                ob.offset[0]*=sc
                ob.offset[1]*=sc
                ob.draw()
                ob.image = i
                ob.offset = off
        else:
            [ob.draw() for ob in self.objects]
        self.foreground.draw()
        
    def find_object(self, pos):
        """Returns front-most object colliding with pos"""
        for ob in reversed(self.objects):
            if not ob.image:
                continue
            corner = [ob.pos[0], ob.pos[1]]
            corner[0]-=ob.offset[0]
            corner[1]-=ob.offset[1]
            if pos[0]>=corner[0] and \
               pos[0]<=corner[0]+ob.image.get_width() and \
               pos[1]>=corner[1] and \
               pos[1]<=corner[1]+ob.image.get_height():
                return ob

    def can_walk(self, pos):
        if self.true(self.walk[int(pos[0])][int(pos[1])]):
            return True

    def add_path(self, object, dests, collision=True):
        path = {"object": object, 
            "dests": dests, "collision": collision}
        self.paths.append(path)
        return path

    def clear_paths(self, object):
        self.paths[:] = [x for x in self.paths if x['object'] != object]

    def animate_paths(self):
        for path in self.paths[:]:
            object, dests = path['object'], path['dests']
            if not dests:
                if hasattr(object, "mainseq"):
                    object.seq = object.mainseq
                if "next" in path:
                    path['next']()
                self.paths.remove(path)
                continue
            dest = dests[0]
            col = path['collision']
            vec = [dest[0]-object.pos[0], dest[1]-object.pos[1]]
            mag = math.sqrt(vec[0]**2+vec[1]**2)
            if mag < 1:
                del path['dests'][0]
                continue
            move = [vec[0]/mag*5, vec[1]/mag*5]
            if abs(move[0])>abs(vec[0]):
                move[0] = vec[0]
            if abs(move[1])>abs(vec[1]):
                move[1] = vec[1]
            #Let him walk if he's already stuck
            if not self.can_walk(object.pos):
                col = False
            op = object.pos[:]
            object.pos[0]+=move[0]
            object.pos[1]+=move[1]
            if col and not self.can_walk(object.pos):
                del path['dests'][0]
                continue
            if abs(object.pos[0]-dest[0])<2 and \
               abs(object.pos[1]-dest[1])<2:
                del path['dests'][0]
            if move[0] < 0 and hasattr(object, "leftseq"):
                object.seq = object.leftseq
                object.flipped = True
            if move[0] > 0 and hasattr(object, "rightseq"):
                object.seq = object.rightseq
                object.flipped = False

    def astar(self, start, goal, smoothness=3):
        s = smoothness
        def hguess(start, end):
            manhat = (end[0]-start[0])**2+(end[1]-start[1])**2
            return math.sqrt(manhat)
        def near(pos):
            l = [(pos[0]-s, pos[1]),(pos[0]+s, pos[1]),
                (pos[0], pos[1]-s), (pos[0], pos[1]+s),
                (pos[0]-s, pos[1]-s), (pos[0]-s, pos[1]+s),
                (pos[0]+s, pos[1]-s), (pos[1]+s, pos[1]+s)]
            return l
        def path(camefrom, goal):
            p = [goal]
            cur = goal
            while camefrom:
                if cur in camefrom:
                    old = cur
                    p.append(camefrom[cur])
                    cur = camefrom[cur]
                    del camefrom[old]
                else:
                    break
            p.reverse()
            return p
        start = tuple([int(x)//s*s for x in start])
        goal = tuple([int(x)//s*s for x in goal])
        closed = set()
        open = [start]
        g = {}
        h = {}
        f = {}
        camefrom = {}
        g[start] = 0
        h[start] = hguess(start, goal)
        f[start] = h[start]
        while open:
            open.sort(key=lambda o: f[o])
            x = open[0]
            if x == goal:
                return path(camefrom,goal)
            open.remove(x)
            closed.add(x)
            for y in near(x):
                if y in closed:
                    continue
                if y[0]<0 or y[0]>=640 or y[1]<0 or y[1]>=480:
                    continue
                if not self.can_walk(y):
                    continue
                tg = g[x]+1   #Cost
                t_better = False
                if y not in open:
                    open.append(y)
                    h[y] = hguess(y, goal)
                    t_better = True
                elif tg < g[y]:
                    t_better = True
                if t_better:
                    camefrom[y] = x
                    g[y] = tg
                    f[y] = g[y]+h[y]
        return False


