import pygame

class State(object):

    clock = pygame.time.Clock()
    next = 3.0
    frames = 0

    def loop(self):
        self.dt = self.clock.tick(30)/1000.0
        self.handle_events()
        self.frames += 1
        self.next -= self.dt
        if self.next <= 0:
            self.next = 3.0
            print self.frames/3.0, "fps"
            self.frames = 0

    def handle_events(self):
        for event in pygame.event.get():
            if event.type == pygame.QUIT:
                self.quit()
            if event.type == pygame.MOUSEBUTTONDOWN:
                if event.button == 1:
                    self.left_mouse(event.pos)
                if event.button == 3:
                    self.right_mouse(event.pos)

    def quit(self):
        print "quitting"
        pygame.state = None
        import sys
        sys.exit()

    def left_mouse(self, pos):
        pass

    def right_mouse(self, pos):
        pass


class Intro(State):
    
    def loop(self):
        super(self.__class__, self).loop()
        pygame.state = Game()

import room
import rooms
import sprite

class Game(State):
    locked = False

    def item(self, name, file, pos, description, pickup=True):
        it = sprite.Sprite(file)
        it.name = name
        self.room.add(it)
        it.click = lambda char: self.say(char, description)
        it.pos = pos
        it.set_feet()
        if pickup:
            it.rightclick = lambda char: rooms.walk_and_pickup(
                    self, char, it)
        return it
        
    def use(self, char, ob):
        self.using_item = ob
        
    def lock(self, *args):
        self.locked = True
        
    def unlock(self, *args):
        self.locked = False
    
    def say(self, char, msg, end=None):
        msg = msg.split("\n")
        char.message = msg.pop(0)
        char.message_time = 3*len(char.message)
        if msg:
            char.when_msg_over = lambda char: self.say(char, "\n".join(msg), end)
        elif end:
            char.when_msg_over = end
        self.last_char = char

    def loop(self):
        rooms.testroom(self)
        self.pointer = pygame.image.load("gfx/pointer.png")
        self.items = []
        pygame.mouse.set_visible(0)
        while 1:
            super(self.__class__, self).loop()
            self.draw_screen()

    def draw_screen(self):
        if hasattr(self, "event"):
            self.event.time -= 1
            if self.event.time <= 0:
                self.event()
                del self.event
        self.room.draw()
        if self.locked:
            pygame.display.flip()
            return
        spot = [0, 0]
        for it in self.items:
            it.pos = spot[:]
            it.draw()
            spot[0]+=40
        mp = pygame.mouse.get_pos()
        if hasattr(self, "using_item"):
            self.using_item.pos = mp
            self.using_item.draw()
        else:
            pygame.screen.blit(self.pointer, mp)
        overt = ""
        if hasattr(self, "using_item"):
            overt = "Left click to cancel"
        ob = self.mouse_over(mp)
        if ob:
            if not hasattr(self, "using_item"):
                overt = "Walk here"
            if hasattr(ob, "name"):
                overt = ob.name
                if hasattr(self, "using_item"):
                    overt = "Use "+self.using_item.name+" with "+ob.name
        if overt:
            txt = pygame.font18.render(overt, 1, [255, 255, 255])
            p = [mp[0], mp[1]+32]
            if p[0]<0:
                p[0] = 0
            if p[1]<0:
                p[0] = 0
            if p[0]+txt.get_width()>640:
                p[0] = 640-txt.get_width()
            if p[1]+txt.get_height()>480:
                p[1] = 480-txt.get_height()
            pygame.screen.blit(txt, p)
        pygame.display.flip()

    def mouse_over(self, pos):
        spot = [0, 0]
        for it in self.items:
            if pos[0]>spot[0] and pos[1]<40 and pos[0]<spot[0]+40:
                return it
            spot[0]+=40
        ob = self.room.find_object(pos)
        if ob:
            return ob
        if self.room.can_walk(pos):
            return self.room

    def left_mouse(self, pos):
        if self.locked:
            return
        if hasattr(self, "using_item"):
            del self.using_item
            return
        ob = self.mouse_over(pos)
        if ob and hasattr(ob, "click"):
            ob.click(self.char)
            return
        if ob == self.room:
            self.room.clear_paths(self.char)
            path = self.room.astar(self.char.pos, pos)
            if not path:
                return
            path = [path[0]]+path[1:-1:3]+[path[-1]]
            self.room.add_path(self.char, path)

    def right_mouse(self, pos):
        if self.locked:
            return
        ob = self.mouse_over(pos)
        if hasattr(self, "using_item"):
            f = getattr(ob, "use_"+self.using_item.name, None)
            if not f:
                f = getattr(ob, "use_"+self.using_item.name.lower(), None)
            if f:
                f(self, self.char, self.using_item, ob)
                del self.using_item
                return
            del self.using_item
            self.say(self.char, "I don't think that will work.")
            return
        if ob and hasattr(ob, "rightclick"):
            ob.rightclick(self.char)
            return
        return
        if ob == self.room:
            self.room.clear_paths(self.char)
            path = self.room.astar(self.char.pos, pos)
            if not path:
                return
            path = [path[0]]+path[1:-1:3]+[path[-1]]
            self.room.add_path(self.char, path)
